#include "document.h"
#include "docwindow-p.h"
#include <moo/macros.h>

using namespace moo;

class moo::DocumentPrivate {
    M_DECLARE_PUBLIC(Document)

    Document::DocumentState doc_state;
    QUrl url;
    mutable QString display_name;
    mutable QString display_basename;
    static int untitled_count;

    void ensure_name() const
    {
        if (display_name.isEmpty())
        {
            if (untitled_count > 0)
                display_basename = QString(Document::tr("Untitled %1")).arg(untitled_count + 1);
            else
                display_basename = QString(Document::tr("Untitled"));
            untitled_count += 1;
            display_name = display_basename;
        }
    }

    bool isUntitled() const { return url.isEmpty(); }
    QString displayName() const { ensure_name(); return display_name; }
    QString displayBasename() const { ensure_name(); return display_basename; }

    DocumentPrivate(Document *p) :
        pub(p),
        doc_state(Document::Idle),
        url(),
        display_name(),
        display_basename()
    {
    }
};

int DocumentPrivate::untitled_count;

Document::Document(QObject *parent) :
    QObject(parent),
    impl(this)
{
}

Document::~Document()
{
}

namespace {
struct BusyCursor {
    BusyCursor() { QApplication::setOverrideCursor(QCursor(Qt::WaitCursor)); }
    ~BusyCursor() { QApplication::restoreOverrideCursor(); }
};
}

bool Document::load(const QUrl &url, QString &error)
{
    setDocumentState(Loading);

    bool success = loadFile(url, error);

    if (success)
    {
        setUrl(url);
        setModified(false);
        RecentMenu::addUrl(url);
    }

    setDocumentState(Idle);
    return success;
}

bool Document::save(const QUrl &url, const QString &type, QString &error)
{
    setDocumentState(Saving);

    bool success = saveFile(url, type, error);

    if (success)
    {
        setUrl(url);
        setModified(false);
        RecentMenu::addUrl(url);
    }

    setDocumentState(Idle);
    return success;
}

void Document::setUrl(const QUrl &url)
{
    if (url == impl->url)
        return;

    if (url.isEmpty())
    {
        impl->url = QUrl();
        impl->display_name = QString();
        impl->display_basename = QString();
    }
    else
    {
        QString localFile = url.toLocalFile();
        if (!localFile.isEmpty())
        {
            QFileInfo fi(localFile);
            localFile = fi.absoluteFilePath();
            impl->url = QUrl::fromLocalFile(localFile);
            impl->display_name = QDir::toNativeSeparators(localFile);
            impl->display_basename = fi.fileName();
        }
        else
        {
            impl->url = url;
            impl->display_name = url.toString();
            impl->display_basename = impl->display_name.split('/').last();
        }
    }

    emit urlChanged(impl->url);
}

void Document::setDocumentState(DocumentState state)
{
    if (impl->doc_state != state)
    {
        impl->doc_state = state;
        emit docStateChanged();
    }
}

Document::DocumentState Document::documentState() const
{
    return impl->doc_state;
}

bool Document::isUntitled() const
{
    return impl->isUntitled();
}

QUrl Document::url() const
{
    return impl->url;
}

QString Document::displayName() const
{
    return impl->displayName();
}

QString Document::displayBasename() const
{
    return impl->displayBasename();
}

bool Document::isEmpty() const
{
    return isUntitled() && !isModified();
}

bool Document::needSave() const
{
    return isModified();
}

// QString Document::documentType() const;
// void Document::setDocumentType(const QString &type);
